@@ -21,6 +21,9 @@
  */
 
 #include <config.h>
+#ifdef XPCOM_GLUE
+#	include <gtkmozembed_glue.cpp>
+#endif
 #include <gtkmozembed.h>
 #include <gtkmozembed_internal.h>
 
@@ -36,9 +39,12 @@
 #include <nsISupportsPrimitives.h>
 #include <nsILocalFile.h>
 #include <nsIDOMMouseEvent.h>
+#include <nsIWebBrowserFind.h>
+#include <nsStringAPI.h>
 
 #include <stdlib.h>
 
+#if !defined(HAVE_GECKO_1_8) && !defined(HAVE_GECKO_1_9)
 #if defined (HAVE_CHROME_NSICHROMEREGISTRYSEA_H)
 #include <chrome/nsIChromeRegistrySea.h>
 #elif defined(MOZ_NSIXULCHROMEREGISTRY_SELECTSKIN)
@@ -49,9 +55,11 @@
 // FIXME: For setting the locale. hopefully gtkmozembed will do itself soon
 #include <nsILocaleService.h>
 #endif
+#endif /* !HAVE_GECKO_1_8 */
 
 #include "dh-util.h"
 #include "dh-gecko-utils.h"
+#include "Yelper.h"
 
 gint
 dh_gecko_utils_get_mouse_event_button (gpointer event)
@@ -121,6 +129,10 @@ dh_util_split_font_string (const gchar *font_name, gchar **name, gint *size)
 	PangoFontMask         mask;
 	gboolean              retval = FALSE;
 
+	if (font_name == NULL) {
+		return FALSE;
+	}
+
 	mask = (PangoFontMask) (PANGO_FONT_MASK_FAMILY | PANGO_FONT_MASK_SIZE);
 	
 	desc = pango_font_description_from_string (font_name);
@@ -139,36 +151,52 @@ dh_util_split_font_string (const gchar *font_name, gchar **name, gint *size)
 	return retval;
 }
 
+static gboolean
+gecko_prefs_set_bool (const gchar *key, gboolean value)
+{
+	nsresult rv;
+	nsCOMPtr<nsIPrefService> prefService (do_GetService (NS_PREFSERVICE_CONTRACTID, &rv));
+	NS_ENSURE_SUCCESS (rv, FALSE);
+
+	nsCOMPtr<nsIPrefBranch> pref;
+	rv = prefService->GetBranch ("", getter_AddRefs (pref));
+	NS_ENSURE_SUCCESS (rv, FALSE);
+
+	rv = pref->SetBoolPref (key, value);
+
+	return NS_SUCCEEDED (rv) != PR_FALSE;
+}
+
 static gboolean
 gecko_prefs_set_string (const gchar *key, const gchar *value)
 {
-	nsCOMPtr<nsIPrefService> prefService =
-		do_GetService (NS_PREFSERVICE_CONTRACTID);
-	nsCOMPtr<nsIPrefBranch> pref;
-	prefService->GetBranch ("", getter_AddRefs (pref));
+	nsresult rv;
+	nsCOMPtr<nsIPrefService> prefService (do_GetService (NS_PREFSERVICE_CONTRACTID, &rv));
+	NS_ENSURE_SUCCESS (rv, FALSE);
 
-	if (pref) {
-		nsresult rv = pref->SetCharPref (key, value);
-		return NS_SUCCEEDED (rv) ? TRUE : FALSE;
-	}
-	
-	return FALSE;
+	nsCOMPtr<nsIPrefBranch> pref;
+	rv = prefService->GetBranch ("", getter_AddRefs (pref));
+	NS_ENSURE_SUCCESS (rv, FALSE);
+
+	rv = pref->SetCharPref (key, value);
+
+	return NS_SUCCEEDED (rv) != PR_FALSE;
 }
 
 static gboolean
 gecko_prefs_set_int (const gchar *key, gint value)
 {
-	nsCOMPtr<nsIPrefService> prefService =
-		do_GetService (NS_PREFSERVICE_CONTRACTID);
-	nsCOMPtr<nsIPrefBranch> pref;
-	prefService->GetBranch ("", getter_AddRefs (pref));
+	nsresult rv;
+	nsCOMPtr<nsIPrefService> prefService (do_GetService (NS_PREFSERVICE_CONTRACTID, &rv));
+	NS_ENSURE_SUCCESS (rv, FALSE);
 
-	if (pref) {
-		nsresult rv = pref->SetIntPref (key, value);
-		return NS_SUCCEEDED (rv) ? TRUE : FALSE;
-	}
-	
-	return FALSE;
+	nsCOMPtr<nsIPrefBranch> pref;
+	rv = prefService->GetBranch ("", getter_AddRefs (pref));
+	NS_ENSURE_SUCCESS (rv, FALSE);
+
+	rv = pref->SetIntPref (key, value);
+
+	return NS_SUCCEEDED (rv) != PR_FALSE;
 }
 
 extern "C" void 
@@ -201,6 +229,8 @@ dh_gecko_utils_set_font (gint type, const gchar *fontname)
 	g_free (name);
 }		   
 
+#if !defined(HAVE_GECKO_1_8) && !defined(HAVE_GECKO_1_9)
+
 static nsresult
 getUILang (nsAString& aUILang)
 {
@@ -259,28 +289,177 @@ gecko_utils_init_chrome (void)
 #endif
 }
 
-extern "C" void
-dh_gecko_utils_init_services (void)
-{
-	gchar *profile_dir;
+#endif /* !HAVE_GECKO_1_8  && !HAVE_GECKO_1_9 */
 
+static nsresult
+gecko_utils_init_prefs (void)
+{
+	nsresult rv;
+	nsCOMPtr<nsIPrefService> prefService (do_GetService (NS_PREFSERVICE_CONTRACTID, &rv));
+	NS_ENSURE_SUCCESS (rv, rv);
+
+	nsCOMPtr<nsILocalFile> file;
+	rv = NS_NewNativeLocalFile (nsEmbedCString (SHAREDIR "/default-prefs.js"),
+				   PR_TRUE, getter_AddRefs (file));
+	NS_ENSURE_SUCCESS (rv, rv);
+
+	rv = prefService->ReadUserPrefs (file);
+	rv |= prefService->ReadUserPrefs (nsnull);
+	NS_ENSURE_SUCCESS (rv, rv);
+
+	return rv;
+}
+
+extern "C" void
+dh_gecko_utils_init (void)
+{
 	if (!g_thread_supported ()) {
 		g_thread_init (NULL);
 	}
 
-	gtk_moz_embed_set_comp_path (MOZILLA_HOME);
-	
-	profile_dir = g_build_filename (g_getenv ("HOME"), 
-					".gnome2",
-					"devhelp",
-					"mozilla", NULL);
+#ifdef HAVE_GECKO_1_9
+	NS_LogInit ();
+#endif
+
+nsresult rv;
+#ifdef XPCOM_GLUE
+    static const GREVersionRange greVersion = {
+    "1.9a", PR_TRUE,
+    "1.9.*", PR_TRUE
+    };
+    char xpcomLocation[4096];
+    rv = GRE_GetGREPathWithProperties(&greVersion, 1, nsnull, 0, xpcomLocation, 4096);
+    if (NS_FAILED (rv))
+    {
+       g_warning ("Could not determine locale!\n");
+       return;
+    }
+
+    // Startup the XPCOM Glue that links us up with XPCOM.
+    rv = XPCOMGlueStartup(xpcomLocation);
+    if (NS_FAILED (rv))
+    {
+       g_warning ("Could not determine locale!\n");
+       return;
+    }
+
+    rv = GTKEmbedGlueStartup();
+    if (NS_FAILED (rv))
+    {
+       g_warning ("Could not startup embed glue!\n");
+       return;
+    }
+
+    rv = GTKEmbedGlueStartupInternal();
+    if (NS_FAILED (rv))
+    {
+       g_warning ("Could not startup embed glue (internal)!\n");
+       return;
+    }
+
+    char *lastSlash = strrchr(xpcomLocation, '/');
+    if (lastSlash)
+      *lastSlash = '\0';
+
+    gtk_moz_embed_set_path(xpcomLocation);
+
+#else
+#ifdef HAVE_GECKO_1_9
+	gtk_moz_embed_set_path (GECKO_HOME);
+#else
+	gtk_moz_embed_set_comp_path (GECKO_HOME);
+#endif
+#endif // XPCOM_GLUE
+
+	gchar *profile_dir = g_build_filename (g_get_home_dir (),
+					       ".gnome2",
+					       "devhelp",
+					       "mozilla", NULL);
 
 	gtk_moz_embed_set_profile_path (profile_dir, "Devhelp");
 	g_free (profile_dir);
 
 	gtk_moz_embed_push_startup ();
-        
-	gecko_prefs_set_string ("font.size.unit", "pt");
+
+	gecko_utils_init_prefs ();
+
+#if !defined(HAVE_GECKO_1_8) && !defined(HAVE_GECKO_1_9)
 	gecko_utils_init_chrome ();
+#endif
 }
 
+extern "C" void
+dh_gecko_utils_shutdown (void)
+{
+	gtk_moz_embed_pop_startup ();
+
+#ifdef HAVE_GECKO_1_9
+	NS_LogTerm ();
+#endif
+}
+
+
+extern "C" gboolean
+dh_gecko_utils_search_find (Yelper *yelper, const gchar * text)
+{
+	yelper->Init();
+	return yelper->Find (text);
+}
+
+extern "C" gboolean
+dh_gecko_utils_search_find_again (Yelper *yelper, gboolean backward)
+{
+	yelper->Init();
+	return yelper->FindAgain(!backward);
+}
+
+extern "C" float
+dh_gecko_utils_get_zoom (Yelper *yelper)
+{
+	nsresult rv;
+	float zoom;
+
+	yelper->Init();
+	rv = yelper->GetZoom (&zoom);
+	
+	if (NS_SUCCEEDED (rv))
+	{
+		return zoom;
+	}
+
+	return 1.0;
+}
+
+extern "C" gboolean
+dh_gecko_utils_set_zoom (Yelper *yelper,
+			 float zoom) 
+{
+	nsresult rv;
+
+	g_return_val_if_fail (zoom > 0.0, FALSE);
+
+	yelper->Init();
+	rv = yelper->SetZoom (zoom);
+
+	return (NS_SUCCEEDED (rv));
+}		
+
+extern "C" void
+dh_gecko_utils_search_set_case_sensitive (Yelper * yelper, gboolean case_sensitive)
+{
+	yelper->Init();
+	yelper->SetFindProperties (NULL, case_sensitive, FALSE);
+}
+
+extern "C" Yelper *
+dh_gecko_utils_create_yelper (GtkMozEmbed *gecko)
+{
+	Yelper *yelper = new Yelper (gecko);
+	return yelper;
+}
+
+extern "C" void
+dh_gecko_utils_destroy_yelper (Yelper *yelper)
+{
+	delete yelper;
+}
